package DBIx::QuickORM::Manual;
use strict;
use warnings;

1;

__END__

=head1 NAME

DBIx::QuickORM::Manual - Documentation hub for L<DBIx::QuickORM>.

=head1 DESCRIPTION

This manual is a hub that can be used to find documentation on
L<DBIx::QuickORM>.

=head1 TUTORIALS, HOWTOS, AND ESSENTIALS

=over 4

=item L<DBIx::QuickORM::Manual::Concepts>

This document covers several key concepts of L<DBIx::QuickORM>. Understanding
these concepts will make it much easier to work with L<DBIx::QuickORM> instead
of fighting against it.

=back

=head2 TODO - More to come

=head1 KEY MODULE DOCS

=over 4

=item L<DBIx::QuickORM>

The L<DBIx::QuickORM> module exports the primary interface for defining an ORM.

=item L<DBIx::QuickORM::Connection>

The L<DBIx::QuickORM::Connection> module is used for each connection you make
to the database. This is where Transactions are controlled and handles are
made.

=item L<DBIx::QuickORM::Handle>

The L<DBIx::QuickORM::Handle> module implements the primary interface for
fetching, creating, and modifying rows in the database.

=back

=head1 SOURCE

The source code repository for DBIx-QuickORM can be found at
L<http://github.com/exodist/DBIx-QuickORM/>.

=head1 MAINTAINERS

=over 4

=item Chad Granum E<lt>exodist@cpan.orgE<gt>

=back

=head1 AUTHORS

=over 4

=item Chad Granum E<lt>exodist@cpan.orgE<gt>

=back

=head1 COPYRIGHT

Copyright Chad Granum E<lt>exodist7@gmail.comE<gt>.

This program is free software; you can redistribute it and/or
modify it under the same terms as Perl itself.

See L<http://dev.perl.org/licenses/>

=cut
